/*
* Copyright (c) 2004-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of the License "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*
*/


#include "fileextractor.h"
#include "progressbar.h"
#include "sishelper.h"
#include <bautils.h>
#include "sisuihandler.h"

#include "sisregistryfiledescription.h"

namespace Swi
{

/*static*/ CFileExtractor* CFileExtractor::NewLC(RFs& aFs, RSisHelper& aSisHelper, RUiHandler& aUiHandler, const TAppInfo& aAppInfo)
	{
	CFileExtractor* self=new(ELeave) CFileExtractor(aFs, aSisHelper, aUiHandler, aAppInfo);
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}
	
/*static*/ CFileExtractor* CFileExtractor::NewL(RFs& aFs, RSisHelper& aSisHelper, RUiHandler& aUiHandler, const TAppInfo& aAppInfo)
	{
	CFileExtractor* self=NewLC(aFs, aSisHelper, aUiHandler, aAppInfo);
	CleanupStack::Pop(self);
	return self;
	}
	
CFileExtractor::~CFileExtractor()
	{
	Deque();
	
	if (iManagedFileHandle)
		{
		iCurrentFile->Close();
		delete iCurrentFile;
		}
	delete iFileDescription;
    delete fMan;
	}

CFileExtractor::CFileExtractor(RFs& aFs, RSisHelper& aSisHelper, RUiHandler& aUiHandler, const TAppInfo& aAppInfo)
	: CActive(EPriorityStandard), iFs(aFs), iSisHelper(aSisHelper), iUiHandler(aUiHandler), iAppInfo(aAppInfo)
	{
	CActiveScheduler::Add(this);
	}

void CFileExtractor::ConstructL()
	{
	User::LeaveIfError(iFs.ShareProtected());
    
    fMan = CFileMan::NewL(iFs); 
    
    _LIT(KTempFolderC, "C:\\installserver\\");        
    _LIT(KTempFolderCDummy, "C:\\installserver\\dummy\\");        
    _LIT(KTempFolderD, "D:\\installserver\\");        
    _LIT(KTempFolderDDummy, "D:\\installserver\\dummy\\");        
    _LIT(KTempFolderE, "E:\\installserver\\");        
    _LIT(KTempFolderEDummy, "E:\\installserver\\dummy\\");        
    _LIT(KTempFolderF, "F:\\installserver\\");        
    _LIT(KTempFolderFDummy, "F:\\installserver\\dummy\\");        

    goCopy = EFalse;
    
    if (iFs.MkDir(KTempFolderFDummy) == KErrNone)
        {
        iFs.RmDir(KTempFolderFDummy);
        TempFolder.Copy(KTempFolderF);
        goCopy = ETrue;
        }
    else if (iFs.MkDir(KTempFolderEDummy) == KErrNone)
        {
        iFs.RmDir(KTempFolderEDummy);
        TempFolder.Copy(KTempFolderE);
        goCopy = ETrue;
        }
    else if (iFs.MkDir(KTempFolderDDummy) == KErrNone)
        {
        iFs.RmDir(KTempFolderDDummy);
        TempFolder.Copy(KTempFolderD);
        goCopy = ETrue;
        }
    else if (iFs.MkDir(KTempFolderCDummy) == KErrNone)
        {
        iFs.RmDir(KTempFolderCDummy);
        TempFolder.Copy(KTempFolderC);
        goCopy = ETrue;
        }
    
    if (goCopy)
        {
        TBuf<255> aFileName;
        TBuf8<255> aFileName8;
        iSisHelper.GetSisFileNameL(aFileName);
    
        TParse parse;
        parse.SetNoWild(aFileName, NULL, NULL);
        TFileName iFileName = parse.Name();
        TFileName iFileNameExt = parse.NameAndExt();

        TFileName iSavePath;
        iSavePath.Copy(TempFolder);
        iSavePath.Append(iFileNameExt);

        TFileName iLogFile;
        iLogFile.Copy(TempFolder);
        iLogFile.Append(_L("installserver.log"));        
        
        RFile aFile; /// 
        TInt aErr = (aFile.Open(iFs, iLogFile, EFileWrite)); 
        if (aErr == KErrNone) 
            { 
            CleanupClosePushL(aFile); /// 
            TInt aSeekPos = 0;
            aFile.Seek(ESeekEnd, aSeekPos);

            aFileName8.Copy(iFileName);
            aFile.Write(aFileName8); 
            aFile.Write(_L8("\n")); 
            CleanupStack::PopAndDestroy();
            
            fMan->Copy(aFileName, iSavePath);
            }
        
        TempFolder.Append(iFileName);
        TempFolder.Append(_L("\\"));
        }
	}

// public methods

void CFileExtractor::ExtractFileL(TInt aDataUnit, const CSisRegistryFileDescription& aFileDescription, const TDesC& aFileName, TRequestStatus& aStatus)
	{
	iManagedFileHandle=ETrue;
	
	iCurrentFile=new(ELeave) RFile;
	User::LeaveIfError(iCurrentFile->Replace(iFs, aFileName, EFileWrite));
	
	ExtractFileL(aDataUnit, aFileDescription, *iCurrentFile, aStatus);
	}


void CFileExtractor::ExtractFileL(TInt aDataUnit, const CSisRegistryFileDescription& aFileDescription, RFile& aFile, TRequestStatus& aStatus)
	{
	iCancelled=EFalse;
	iClientStatus=&aStatus;
	iCurrentFile=&aFile;
	iBytesToExtract=aFileDescription.UncompressedLength();
	iDataUnit=aDataUnit;
	
	delete iFileDescription;
	iFileDescription=0;
	iFileDescription=CSisRegistryFileDescription::NewL(aFileDescription);
	
	
	*iClientStatus=KRequestPending;
	
	iSisHelper.SetupAsyncExtractionL(iFs, *iCurrentFile, iFileDescription->Index(), iDataUnit);

	TRequestStatus* status=&iStatus;
	User::RequestComplete(status, KErrNone);
	SetActive();
	}
		
	
TInt CFileExtractor::FinishExtraction(TInt aResult)
	{
    TBuf<255> aSourceName;
    iCurrentFile->FullName(aSourceName);
        
    TBuf16<255> iTarget;
    iTarget = iFileDescription->Target(); 
    
	TRAPD(err, iSisHelper.EndAsyncExtractionL());
	if (err != KErrNone)
		return err;

	if (iManagedFileHandle)
		{        
		iCurrentFile->Close();
		delete iCurrentFile;
		iCurrentFile=0;
		iManagedFileHandle = EFalse;   
		}
        
    TBuf<255> iFinalName;
        
    TFileName partName;
    partName.Append(iTarget);    
    partName.Delete(partName.Find(_L(":")),1);
    
    if (goCopy)
        {       
        iFinalName.Copy(TempFolder);
        iFinalName.Append(partName);
        TParse parse;
        parse.SetNoWild(iFinalName, NULL, NULL);
        TFileName iFinalPath = parse.DriveAndPath();
    
        iFs.MkDirAll(iFinalPath);
        
        fMan->Copy(aSourceName, iFinalName); 
        } 
		
	if (aResult==KErrNone)
		{  
		// The progress bar should always be incremented at the end of a successful extraction             
		TRAPD(err, iUiHandler.UpdateProgressBarL(iAppInfo, KProgressBarEndIncrement));
		if (err != KErrNone)
			return err;
		}
		
	// notify the user
	User::RequestComplete(iClientStatus, aResult);
	iClientStatus = NULL;
	return KErrNone;
	}

// CActive methods

void CFileExtractor::RunL()
	{
	if (iStatus.Int() != KErrNone)
		{
		User::LeaveIfError(FinishExtraction(iStatus.Int()));
		return;
		}
	
	if (iCancelled)
		{
		User::LeaveIfError(FinishExtraction(KErrCancel));
		return;
		}

	TInt32 chunkSize=iBytesToExtract > KProgressBarInstallChunkSize ? KProgressBarInstallChunkSize : iBytesToExtract; 	

	if (chunkSize==0)
		{
		User::LeaveIfError(FinishExtraction(KErrNone));
		return;
		}

	// Extract the next chunk
	iUiHandler.UpdateProgressBarL(iAppInfo, ProgressBarChunkIncrement(chunkSize));
	iSisHelper.AsyncExtractionL(chunkSize, iStatus);
	iBytesToExtract-=chunkSize;
	SetActive();
	}

void CFileExtractor::DoCancel()
	{
	iCancelled=ETrue;
	if (iClientStatus)
		{
		TInt err = FinishExtraction(KErrCancel);
		if (err != KErrNone)
			{
			User::RequestComplete(iClientStatus, KErrCancel);
			iClientStatus=NULL;
			}
		}
	}
	
TInt CFileExtractor::RunError(TInt aError)
	{
	iCancelled = ETrue;
	if (iClientStatus)
		{
		TInt err = FinishExtraction(aError);
		if (err != KErrNone)
			{
			User::RequestComplete(iClientStatus, aError);
			iClientStatus = NULL;
			}
		}
	return KErrNone;
	}
	

}

